/* libupnpp/config.h.  Generated from config.h.in by configure.  */
/* libupnpp/config.h.in.  Generated from configure.ac by autoheader.  */

/* Define to 1 if you have the <dlfcn.h> header file. */
#define HAVE_DLFCN_H 1

/* Define to 1 if you have the `getifaddrs' function. */
#define HAVE_GETIFADDRS 1

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* Define to 1 if you have the `pthread' library (-lpthread). */
#define HAVE_LIBPTHREAD 1

/* Define to 1 if you have the `rt' library (-lrt). */
/* #undef HAVE_LIBRT */

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the `std::future`. */
#define HAVE_STD_FUTURE 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Define to 1 if you have the `UpnpSetLogLevel' function. */
/* #undef HAVE_UPNPSETLOGLEVEL */

/* Define to the sub-directory where libtool stores uninstalled libraries. */
#define LT_OBJDIR ".libs/"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "jfd@lesbonscomptes.com"

/* Define to the full name of this package. */
#define PACKAGE_NAME "libupnpp"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "libupnpp 0.17.1"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "libupnpp"

/* Define to the home page for this package. */
#define PACKAGE_URL "http://www.lesbonscomptes.com/upmpdcli"

/* Define to the version of this package. */
#define PACKAGE_VERSION "0.17.1"

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* File Offset size */
#define _FILE_OFFSET_BITS 64

/* Large files support */
#define _LARGE_FILE_SOURCE /**/

/* Define to `long int' if <sys/types.h> does not define. */
/* #undef off_t */

/* Define to `unsigned int' if <sys/types.h> does not define. */
/* #undef size_t */

#include "libupnpp/conf_post.h"
